//
//  FilterByTypePreferenceView.swift
//  SampleApp
//
//  Created by Manuchet Rungraksa on 12/10/2565 BE.
//  Copyright © 2565 BE David Zhang. All rights reserved.
//
import SwiftUI
import AmitySDK

struct FilterByTypePreferenceView: View {
    @State var messageId = ""
    @State var messageInfo = ""
    
    let channelId: String
    
    @State var filteredMessageTypes: Int? {
        didSet {
            if filteredMessageTypes == nil {
                UserDefaults.standard.channelPreferenceFilterByType[channelId] = nil
            } else {
                UserDefaults.standard.channelPreferenceFilterByType[channelId] = filteredMessageTypes

            }
        }
    }
    
    let messageType = AmityMessageType.allCases
    
    let viewModel = MessageQueryViewModel()
    
    var body: some View {
        VStack {
            Section() {
                List {
                    ForEach(messageType, id: \.self) { item in
                        MultipleSelectionRow(title: item.description, isSelected: filteredMessageTypes == item.rawValue ) {
                            if filteredMessageTypes == item.rawValue {
                                filteredMessageTypes = nil
                            } else {
                                filteredMessageTypes = item.rawValue
                            }
                        }
                    }
                }
            }
        }.padding()
            .navigationBarTitle("Message Types", displayMode: .inline)
        
    }
}

struct FilterByTypePreferenceView_Previews: PreviewProvider {
    static var previews: some View {
        FilterByTypePreferenceView(channelId: "", filteredMessageTypes: nil)
    }
}

extension AmityMessageType: CaseIterable {
    
    public static var allCases: [AmityMessageType] {
        return [.text, .audio, .image, .video, .file, .custom]
    }
    
}
