//
//  ReachUserListView.swift
//  SampleApp
//
//  Created by Manuchet Rungraksa on 27/9/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct ReachUserListView: View {
    
    @ObservedObject var viewModel: ReachUserListViewModel
    
    @State private var isReadUser = true
    
    var body: some View{
        VStack {
            HStack {
                Text("Reach: \(viewModel.reachCount)")
                Text("Impression \(viewModel.impressionCount)")
            }
            
            
        }.padding([.leading, .trailing], 30)
            .padding(.top, 15)
        
        Divider()
        
        UserListView(usersCollection: viewModel.usersCollection, viewModel: viewModel)
        
    }
    
}

struct ReachUserListView_Previews: PreviewProvider {
    
    static var previews: some View {
        Text("Read/Deliver user view")
    }
    
}

struct ReachUserListModel: Identifiable {
    var id: String
    
    let userObject: AmityUser
    
    let userId: String
    let displayName: String
    let isDeleted: Bool

    init(user: AmityUser) {
        userObject = user
        id = user.userId
        userId = user.userId
        displayName = user.displayName ?? ""
        isDeleted = user.isDeleted
    }
    
}

struct UserListView: View {
    @ObservedObject var usersCollection: AmityCollection<AmityUser>
    var viewModel: ReachUserListViewModel

    var body: some View {
        List(usersCollection.snapshots, id: \.userId) { item in
            VStack(alignment: .leading) {
                Text("User Id: \(item.userId)").frame(alignment: .leadingFirstTextBaseline)
                Text("DisplayName: \(item.displayName ?? "")")
                Text("isDeleted \(item.isDeleted)" as String)
            }.onAppear {
                if usersCollection.snapshots.last?.userId == item.userId {
                    self.viewModel.fetchNextPage()
                }
            }
        }
    }
}


// Concrete implementor
class ReachUserListViewModel: ObservableObject {
    
    var pageTitle: String
    var reachCount: Int
    var impressionCount: Int
        
    var token: AmityNotificationToken?
    let usersCollection: AmityCollection<AmityUser>
        
    @Published var users: [ReadDeliverUserListModel] = []
    
    var readMemberFilter: [ReadMemberFilterOptions]?
    var deliveredMemberFilter: [DeliveredMemberFilterOptions]?
    
    init(viewedId: String, viewedType: AmityViewedType, impressionCount: Int, reachCount: Int) {
        self.pageTitle = "User list"
        self.impressionCount = impressionCount
        self.reachCount = reachCount
        let userRepo = AmityUserRepository(client: AmityManager.shared.client!)
        self.usersCollection = userRepo.getReachedUsers(viewedType: viewedType, viewedId: viewedId)
    }
    
    func fetchNextPage() {
        guard usersCollection.hasNext else { return }
        usersCollection.nextPage()
    }
    
}
