//
//  GetStoryTargetsView.swift
//  SampleApp
//
//  Created by Manuchet Rungraksa on 12/1/2567 BE.
//  Copyright © 2567 BE David Zhang. All rights reserved.
//

import Foundation
import AmitySDK
import SwiftUI

struct GetStoryTargetsView: View {
    
    @ObservedObject var viewModel: StoryTargetListViewModel
    
    var body: some View {
        VStack(alignment: .center) {
            Text("Format is \"targetType/targetId\" per search target and separate by \", \" (comma and space)")
                .font(.caption)
            TextField("Search by targetType/targetId", text: $viewModel.textTarget)
                .textFieldStyle(RoundedBorderTextFieldStyle())
                .font(.callout)
            
            
            Button(action: {
                viewModel.queryStoryTargets()
            }){
                Text("Query user list")
                    .frame(alignment: .center)
            }.buttonStyle(BorderlessButtonStyle())
            
            if let errorString = viewModel.errorString {
                Text(errorString)
            }
            List(viewModel.storyTargets) { target in
                VStack(alignment: .leading) {
                    Text("Target Id: \(target.targetId)").frame(alignment: .leadingFirstTextBaseline)
                    Text("Target type: \(target.targetType)")
                    Text("Has Unseen: \(target.hasUnseen)" as String)
                }
            }
        }
    }
}

class StoryTargetListViewModel: ObservableObject {
    
    var pageTitle: String
    
    @Published var textTarget: String = ""
    @Published var errorString: String?
        
    private var token: AmityNotificationToken?
    private var storyTargetCollection: AmityCollection<AmityStoryTarget>?
    private var storyRepository = AmityStoryRepository(client: AmityManager.shared.client!)
            
    @Published var storyTargets: [StoryTargetListModel] = []
    
    init() {
        self.pageTitle = "User list"
        
    }
    
    @MainActor 
    func queryStoryTargets() {
        var searchTargets = [StoryTargetSearchInfo]()
        let eachTarget = textTarget.components(separatedBy: ", ")
        for target in eachTarget {
            let splited = target.components(separatedBy: "/")
            
            let searchTarget = StoryTargetSearchInfo(targetType: AmityStoryTargetType(rawValue: splited.first ?? "") ?? .community, targetId: splited.last ?? "")
            searchTargets.append(searchTarget)
        }
        
        storyTargets.removeAll()
        errorString = nil
        token?.invalidate()
        
        storyTargetCollection = storyRepository.getStoryTargets(targets: searchTargets)
        token = storyTargetCollection?.observe({ [weak self] collection, _, error in
            
            if let error {
                self?.errorString = error.localizedDescription
            }
            
            guard let self = self else { return}
            
            var targets = [StoryTargetListModel]()
            for storyTarget in collection.snapshots {
                let model = StoryTargetListModel(storyTarget: storyTarget)
                targets.append(model)
            }
            self.storyTargets = targets
            
        })
        
    }

    
}

struct StoryTargetListModel: Identifiable {
    var id: String
    
    let storyTarget: AmityStoryTarget
    
    let targetType: String
    let targetId: String
    let hasUnseen: Bool
    
    init(storyTarget: AmityStoryTarget) {
        self.storyTarget = storyTarget        
        self.targetType = storyTarget.targetType
        self.targetId = storyTarget.targetId
        self.id = storyTarget.targetId
        self.hasUnseen = storyTarget.hasUnseen
    }
}
