//
//  GetStoryView.swift
//  SampleApp
//
//  Created by Manuchet Rungraksa on 15/1/2567 BE.
//  Copyright © 2567 BE David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct GetStoryView: View {
    @State private var storyId: String = ""
    @StateObject var viewModel = GetStoryViewModel()
    
    var body: some View {
        Form {
            TextField("Enter Story Id", text: $storyId)
            
            Section {
                Button {
                    guard !storyId.isEmpty else {
                        viewModel.result = ""
                        return
                    }
                    viewModel.getStory(id: storyId)
                    
                } label: {
                    HStack {
                        Spacer()
                        Text("Get Story")
                        Spacer()
                    }
                }
            }
            
            Section(header: Text("Here you will see result received from sdk")) {
                VStack(alignment: .leading) {
                    Text("Result")
                        .font(.system(size: 18))
                        .fontWeight(.bold)
                        .padding(.bottom, 12)
                    Text(viewModel.result)
                }
            }
        }
        .navigationTitle("Get Comment")
    }
}


class GetStoryViewModel: ObservableObject {
    @Published var result: String = ""
    
    var token: AmityNotificationToken?
    var storyRepo = AmityStoryRepository(client: AmityManager.shared.client!)
    
    @MainActor
    func getStory(id: String) {
        token = storyRepo.getStory(storyId: id).observe { [weak self] liveObject, error in
            guard let weakSelf = self else {
                return
            }
            
            if error != nil {
                weakSelf.result = "Error when getting story"
                weakSelf.token?.invalidate()
                return
            }
            
            if let object = liveObject.snapshot {
                var textData: String = ""
                
                let model = StoryModel(story: object)
                
                
                for data in model.getPreviewData() {
                    textData += "\(data.key): \(data.value)\n"
                }
                
                weakSelf.result = textData
            }
        }
    }
}
