//
//  SinglePostQueryView.swift
//  SampleApp
//
//  Created by Nishan Niraula on 22/7/2565 BE.
//  Copyright © 2565 BE David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct SinglePostQueryView: View {
    
    @State private var postId: String = ""
    @StateObject var viewModel = SinglePostQueryViewModel()
    
    var body: some View {
        VStack(spacing: 20) {
            TextField("Enter Post Id", text: $postId)
                .textFieldStyle(RoundedBorderTextFieldStyle())
            
            Button {
                guard !postId.isEmpty else { return }
                
                viewModel.fetchPost(id: postId)
                
            } label: {
                Text("Fetch Post")
                    .frame(width: 250, height: 44)
                    .foregroundColor(.white)
                    .background(Color.blue)
                    .cornerRadius(16)
            }
            
            Text(viewModel.postDescription)
                .padding(.top, 32)
            
            Spacer()
        }
        .padding()
    }
}

struct SinglePostQueryView_Previews: PreviewProvider {
    static var previews: some View {
        SinglePostQueryView()
    }
}

class SinglePostQueryViewModel: ObservableObject {
    
    @Published var postDescription: String = "Fetching Posts..."
    
    var token: AmityNotificationToken?
    var postRepo = AmityPostRepository(client: AmityManager.shared.client!)
    
    func fetchPost(id: String) {

        postDescription = "Fetching Posts..."

        token = postRepo.getPost(withId: id).observe({ livePost, error in
            guard let post = livePost.snapshot else { return }

            let dataStatus = livePost.dataStatus.description
            let postDescription = "Data Status: \(dataStatus)\n \(post.modelDescription)"

            self.postDescription = postDescription
        })
    }
}
