//
//  PresenceManager.swift
//  SampleApp
//
//  Created by Amity on 13/6/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import Foundation
import AmitySDK

class PresenceManager {
    
    let client = AmityManager.shared.client!
    let userPresence: AmityUserPresenceRepository
    let channelPresence: AmityChannelPresenceRepository
    
    var snapshot: OnlineUsersSnapshot?
    
    init() {
        userPresence = AmityUserPresenceRepository(client: client)
        channelPresence = AmityChannelPresenceRepository(client: client)
    }
    
    @MainActor
    func startHeartbeat() async throws {
        return try await client.presence.startHeartbeat()
    }
    
    @MainActor
    func stopHeartbeat() {
        client.presence.stopHeartbeat()
    }
    
    func togglePresence(enable: Bool) async throws {
        do {
            if enable {
                try await client.presence.enable()
            } else {
                try await client.presence.disable()
            }
        } catch let error {
            throw error
        }
    }
    
    func fetchUserPresenceState() async throws -> Bool {
        return try await client.presence.isEnabled()
    }
    
    @MainActor
    func getOnlineUserCount() async throws -> Int {
        return try await userPresence.getOnlineUsersCount()
    }
    
    @MainActor
    func getOnlineUsers() async throws {
        self.snapshot = try await userPresence.getOnlineUsersSnapshot()
    }
}
